@@ -563,6 +563,25 @@ const char *debListParser::ParseDepends(const char *Start,const char *Stop,
 
    return res;
 }
+#if APT_PKG_ABI <= 600
+const char *debListParser::ParseDepends(const char *Start,const char *Stop,
+					APT::StringView &Package,APT::StringView &Ver,
+					unsigned int &Op, bool ParseArchFlags,
+					bool const StripMultiArch,
+					bool const ParseRestrictionsList,
+				        string Arch)
+{
+   string_view PackageView;
+   string_view VerView;
+
+   auto res = ParseDepends(Start, Stop, PackageView, VerView, Op, (bool)ParseArchFlags,
+   (bool) StripMultiArch, (bool) ParseRestrictionsList, Arch);
+   Package = PackageView;
+   Ver = VerView;
+
+   return res;
+}
+#endif
 
 const char *debListParser::ParseDepends(const char *Start, const char *Stop,
 					string_view &Package, string_view &Ver,
@@ -846,7 +865,7 @@ bool debListParser::ParseDepends(pkgCache::VerIterator &Ver,
 	 // … but this is probably the best thing to do anyway
 	 if (Package.substr(found + 1) == "native")
 	 {
-	    std::string const Pkg = (std::string{Package.substr(0, found)} += ':') += Ver.Cache()->NativeArch();
+	    std::string const Pkg = std::string{Package, 0, found}.append(":").append(Ver.Cache()->NativeArch());
 	    if (NewDepends(Ver, Pkg, "any", Version, Op | pkgCache::Dep::ArchSpecific, Type) == false)
 	       return false;
 	 }
